# IPR∘CS Backend

## Presentation

IPROCS' backend in rust with a postgres database

## Installation

- Install rust and cargo using [rustup](https://rustup.rs/).

- Install [postgres](https://www.postgresql.org/)

- Install diesel_cli with postgres feature

```
cargo install diesel_cli --no-default-features --features postgres
```

- Clone the project and navigate to its location:

```
cd iprocs
```

- set the environment variable <strong>DATABASE_URL</strong> to connect to the postgres database, this can be set up in a  `.env` file in the iprocs folder:

```
DATABASE_URL=postgres://username:password@localhost/iprocs
```

- Set up the database tables using diesel

```
(cd model && diesel migration run)
```

- Compile and run the project from the back folder

```
cargo run
```

## Test

- To run the tests, navigate to the iprocs folder and run them:

```
cargo test -all
```

- You can specify the workspace you want to run the test on using the -p option:

```
cargo test -p model
```

## Job configuration file

In order to use a job with this tool, a job configuration file must be created. The template of this file is given just bellow.

```yaml
---
bin: String  # Job binary path
parameters:
  - name: String # Parameter name
    value_type: String in ["string", "boolean", "integer", "float", "none"] (default = "string") # Expected value type
    default_value: <value> (default = None) # Default value, need to be of value type
    comment: String (default = None) # Eventual comment on the paramete
    required: Boolean (default = False) # Is the parameter required for the job to work properly
    named: Boolean (default = False) # Is the parameter named
	parameter-name: String (default = None) # Parameter name if it's named
  - #Add as many parameters as needed
```

All the job configuration files will be organized in different folders just as follow:

- root
| - software n°1
| | - version n°1
| | | - job 1
| | | - job 2
| | | - job 3
| | - version n°2
| | | - job 1
| | | - job 2
| | | - job 3

The root folder must be specified in a .env file, located in the project root installation folder, with a variable named ADAPTERS_CONFIG_FOLDER so the iprocs software will be able to get all the different directories containing the job configurations.

The different generated sbatch will be stored in a directory with the same structure that the one for the configuration files, whose path is stored in the 
SBATCH_FOLDER environnement variable. The scripts will take as name the timestamp of when they were generated.

## Database model

The IPROC's database is built as exposed in the following schema.

![](doc/assets/bdd_iprocs.jpeg)


