use schemars::JsonSchema;
use serde::{Deserialize, Serialize};
use uuid::Uuid;

#[derive(Serialize, Deserialize, JsonSchema)]
pub struct JobGroup {
    pub id: Uuid,
    pub input_job_id: Uuid,
    pub workspace_id: Uuid
}

#[derive(Serialize, Deserialize, JsonSchema)]
pub struct InsertJobGroup {
    pub input_job_id: Uuid,
    pub workspace_id: Uuid
}

#[derive(Serialize, Deserialize, JsonSchema)]
pub struct UpdateJobGroup {
    pub input_job_id: Option<Uuid>,
    pub workspace_id: Option<Uuid>
}

impl From<model::models::jobgroups::JobGroup> for JobGroup {
    fn from(j: model::models::jobgroups::JobGroup) -> Self {
        Self {
            id: j.id,
            input_job_id: j.input_job_id,
            workspace_id: j.workspace_id
        }
    }
}

impl From<UpdateJobGroup> for model::models::jobgroups::UpdateJobGroup {
    fn from(j: UpdateJobGroup) -> Self {
        Self {
            input_job_id: j.input_job_id,
            workspace_id: j.workspace_id
        }
    }
}

impl From<InsertJobGroup> for model::models::jobgroups::NewJobGroup {
    fn from(j: InsertJobGroup) -> Self {
        Self {
            input_job_id: j.input_job_id,
            workspace_id: j.workspace_id
        }
    }
}
