use crate::Error;
use schemars::JsonSchema;
use serde::{Deserialize, Serialize};
use uuid::Uuid;
use std::collections::HashMap;

#[derive(Serialize, Deserialize, JsonSchema)]
pub struct CreateJob {
    pub job: InsertJob,
    pub params: HashMap<String, String>,
    pub hardware: HashMap<String, String>
}

#[derive(Serialize, Deserialize, JsonSchema)]
pub struct Job {
    pub id: Uuid,
    pub slurm_id: String,
    pub name: String,
    pub output: String,
    pub job_type: String,
    pub version: String,
    pub software: String
}

#[derive(Serialize, Deserialize, JsonSchema)]
pub struct InsertJob {
    pub name: String,
    pub output: String,
    pub job_type: String,
    pub version: String,
    pub software: String
}

#[derive(Serialize, Deserialize, JsonSchema)]
pub struct UpdateJob {
    pub name: Option<String>,
    pub output: Option<String>,
    pub job_type: Option<String>,
    pub version: Option<String>,
    pub software: Option<String>
}

impl TryFrom<InsertJob> for model::models::jobs::NewJob {
    type Error = Error;
    fn try_from(value: InsertJob) -> Result<Self, Self::Error> {
        Ok(Self {
            slurm_id: "".to_string(),
            name: value.name,
            output: value.output,
            job_type: value.job_type,
            version: value.version,
            software: value.software
        })
    }
}

impl From<UpdateJob> for model::models::jobs::UpdateJob {
    fn from(j: UpdateJob) -> Self {
        Self {
            name: j.name,
            output: j.output,
            job_type: j.job_type,
            version: j.version,
            software: j.software
        }
    }
}

impl From<model::models::jobs::Job> for Job {
    fn from(j: model::models::jobs::Job) -> Self {
        Self {
            id: j.id,
            slurm_id: j.slurm_id,
            name: j.name,
            output: j.output,
            job_type: j.job_type,
            version: j.version,
            software: j.software
        }
    }
}
