use schemars::JsonSchema;
use serde::{Deserialize, Serialize};
use uuid::Uuid;

#[derive(Serialize, Deserialize, JsonSchema)]
pub struct Param {
    pub id: Uuid,
    pub job_id: Uuid,
    pub name: String,
    pub value: String
}

#[derive(Serialize, Deserialize, JsonSchema)]
pub struct InsertParam {
    pub job_id: Uuid,
    pub name: String,
    pub value: String
}

#[derive(Serialize, Deserialize, JsonSchema)]
pub struct GetParam {
    pub id: Uuid,
    pub job_id: Uuid,
    pub name: String,
    pub value: String
}

#[derive(Serialize, Deserialize, JsonSchema)]
pub struct UpdateParam {
    pub job_id: Option<Uuid>,
    pub name: Option<String>,
    pub value: Option<String>
}

impl From<InsertParam> for model::models::parameters::NewParameter {
    fn from(p: InsertParam) -> Self {
        Self {
            job_id: p.job_id,
            name: p.name,
            value: p.value
        }
    }
}

impl From<UpdateParam> for model::models::parameters::UpdateParameter {
    fn from(p: UpdateParam) -> Self {
        Self {
            job_id: p.job_id,
            name: p.name,
            value: p.value
        }
    }
}

impl From<model::models::parameters::UpdateParameter> for UpdateParam {
    fn from(p: model::models::parameters::UpdateParameter) -> Self {
        Self {
            job_id: p.job_id,
            name: p.name,
            value: p.value
        }
    }
}

impl From<model::models::parameters::Parameter> for Param {
    fn from(p: model::models::parameters::Parameter) -> Self {
        Self {
            id: p.id,
            job_id: p.job_id,
            name: p.name,
            value: p.value
        }
    }
}
