use schemars::JsonSchema;
use serde::{Deserialize, Serialize};
use uuid::Uuid;

#[derive(Serialize, Deserialize, JsonSchema)]
pub struct Project {
    pub id: Uuid,
    pub name: String,
    pub user_id: Uuid
}

#[derive(Serialize, Deserialize, JsonSchema)]
pub struct InsertProject {
    pub name: String,
    pub user_id: Uuid
}

#[derive(Serialize, Deserialize, JsonSchema)]
pub struct UpdateProject {
    pub name: Option<String>,
    pub user_id: Option<Uuid>
}

impl From<model::models::projects::Project> for Project {
    fn from(p: model::models::projects::Project) -> Self {
        Self {
            id: p.id,
            name: p.name,
            user_id: p.user_id
        }
    }
}

impl From<InsertProject> for model::models::projects::NewProject {
    fn from(p: InsertProject) -> Self {
        Self {
            name: p.name,
            user_id: p.user_id
        }
    }
}

impl From<UpdateProject> for model::models::projects::UpdateProject {
    fn from(p: UpdateProject) -> Self {
        Self {
            name: p.name,
            user_id: p.user_id
        }
    }
}
