use schemars::JsonSchema;
use serde::{Deserialize, Serialize};
use uuid::Uuid;

#[derive(Serialize, Deserialize, JsonSchema)]
pub struct User {
    pub id: Uuid,
    pub team: String,
    pub identity: String
}

#[derive(Serialize, Deserialize, JsonSchema)]
pub struct InsertUser {
    pub team: String,
    pub identity: Option<String>
}

#[derive(Serialize, Deserialize, JsonSchema)]
pub struct UpdateUser {
    pub team: Option<String>,
    pub identity: Option<String>
}

impl From<model::models::users::User> for User {
    fn from(u: model::models::users::User) -> Self {
        Self {
            id: u.id,
            team: u.team,
            identity: match u.identity {
                Some(identity) => identity,
                None => "".to_string()
            }
        }
    }
}

impl From<InsertUser> for model::models::users::NewUser {
    fn from(u: InsertUser) -> Self {
        Self {
            team: u.team,
            identity: u.identity
        }
    }
}

impl From<UpdateUser> for model::models::users::UpdateUser {
    fn from(u: UpdateUser) -> Self {
        Self {
            team: u.team,
            identity: u.identity
        }
    }
}
