use schemars::JsonSchema;
use serde::{Deserialize, Serialize};
use uuid::Uuid;

#[derive(Serialize, Deserialize, JsonSchema)]
pub struct Workspace {
    pub id: Uuid,
    pub name: String,
    pub project_id: Uuid
}

#[derive(Serialize, Deserialize, JsonSchema)]
pub struct InsertWorkspace {
    pub name: String,
    pub project_id: Uuid
}

#[derive(Serialize, Deserialize, JsonSchema)]
pub struct GetWorkspace {
    pub id: Uuid,
    pub name: String,
    pub project_id: Uuid
}

#[derive(Serialize, Deserialize, JsonSchema)]
pub struct UpdateWorkspace {
    pub name: Option<String>,
    pub project_id: Option<Uuid>
}

impl From<model::models::workspaces::Workspace> for Workspace {
    fn from(w: model::models::workspaces::Workspace) -> Self {
        Self {
            id: w.id,
            name: w.name,
            project_id: w.project_id
        }
    }
}

impl From<InsertWorkspace> for model::models::workspaces::NewWorkspace {
    fn from(w: InsertWorkspace) -> Self {
        Self {
            name: w.name,
            project_id: w.project_id
        }
    }
}

impl From<UpdateWorkspace> for model::models::workspaces::UpdateWorkspace {
    fn from(w: UpdateWorkspace) -> Self {
        Self {
            name: w.name,
            project_id: w.project_id
        }
    }
}
