#[macro_use]
extern crate rocket;
extern crate dotenv;

use std::env;
use std::path::Path;

use dotenv::dotenv;
use rocket_okapi::openapi_get_routes;
use adapters::adapters::load_conf_files;
use router::cors::CORS;

#[launch]
#[cfg(not(tarpaulin_include))]
fn rocket() -> _ {
    dotenv().ok();
    // Getting config directory set as a en env var
    let config_dir = env::var("ADAPTERS_CONFIG_FOLDER").unwrap();
    // Getting sbatch directory set as a en env var to see if it's set
    let _sbtach_dir = env::var("SBATCH_FOLDER").unwrap();
    // Loading and checking every software configuration file
    let parsed_configurations = load_conf_files(Path::new(&config_dir)).unwrap();
    rocket::build()
        // Make the parsed configurations available for each routes
        .manage(parsed_configurations)
        .attach(CORS)
        .mount(
            "/teams",
            openapi_get_routes![
                router::routes::teams::get_team_project_list,
                router::routes::teams::get_user_in_team,
            ]
        )
        .mount(
            "/users",
            openapi_get_routes![
                router::routes::users::create_user,
                router::routes::users::get_user_list,
                router::routes::users::get_projects,
                router::routes::users::get_user,
                router::routes::users::update_user,
                router::routes::users::delete_user,
            ],
        )
        .mount(
            "/projects",
            openapi_get_routes![
                router::routes::projects::create_project,
                router::routes::projects::get_workspaces,
                router::routes::projects::get_project,
                router::routes::projects::update_project,
                router::routes::projects::delete_project,
                router::routes::projects::option_projects,
                router::routes::projects::option_get_project
            ],
        )
        .mount(
            "/workspaces",
            openapi_get_routes![
                router::routes::workspaces::create_workspace,
                router::routes::workspaces::get_groups,
                router::routes::workspaces::get_workspace,
                router::routes::workspaces::update_workspace,
                router::routes::workspaces::delete_workspace,
            ],
        )
        .mount(
            "/jobgroups",
            openapi_get_routes![
                router::routes::jobgroups::create_jobgroup,
                router::routes::jobgroups::get_jobs,
                router::routes::jobgroups::get_nodes,
                router::routes::jobgroups::get_jobgroup,
                router::routes::jobgroups::update_jobgroup,
                router::routes::jobgroups::delete_jobgroup,
            ],
        )
        .mount(
            "/jobs",
            openapi_get_routes![
                router::routes::jobs::create_job,
                router::routes::jobs::get_parameters,
                router::routes::jobs::get_job,
                router::routes::jobs::update_job,
                router::routes::jobs::delete_job,
            ],
        )
        .mount(
            "/nodes",
            openapi_get_routes![
                router::routes::nodes::create_node,
                router::routes::nodes::get_node,
                router::routes::nodes::update_node,
                router::routes::nodes::delete_node,
            ],
        )
        .mount(
            "/parameters",
            openapi_get_routes![
                router::routes::parameters::create_parameter,
                router::routes::parameters::get_parameter,
                router::routes::parameters::update_param,
                router::routes::parameters::delete_param,
            ],
        )
        .mount(
            "/keys",
            openapi_get_routes![
                router::routes::keys::new_key,
                router::routes::keys::delete_key,
                router::routes::keys::get_key_associated_user,
            ],
        )
}

// curl -X POST http://localhost:8000/projects -H 'Content-Type: application/json' -d '{"name":"hello"}'
