use std::env;
use std::fmt;
use std::io::Error as IOError;
use tera;

#[derive(Debug)]
pub enum MiddlewareError {
    ModelError(diesel::result::Error),
    ParseError(dto::Error),
    ConfigError(String),
    CommandError(adapters::constants::CommandError),
    GenerateSbatchError(tera::Error),
    SbatchError(IOError),
    EnVarError(env::VarError)
}

#[cfg(not(tarpaulin_include))]
impl fmt::Display for MiddlewareError {
    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
        match self {
            MiddlewareError::ModelError(_) => {
                write!(f, "An error happened while trying to connect to the database")
            },
            MiddlewareError::ParseError(_) => {
                write!(f, "An error happened while parsing from a struct to another")
            },
            MiddlewareError::ConfigError(msg) => {
                write!(f, "{}", msg)
            },
            MiddlewareError::CommandError(_) => {
                write!(f, "An error happened while generating the job command")
            },
            MiddlewareError::GenerateSbatchError(_) => {
                write!(f, "An error happened while generating the job's slurm batch script")
            },
            MiddlewareError::SbatchError(_) => {
                write!(f, "An error happened while running the job's slurm batch script")
            },
            MiddlewareError::EnVarError(_) => {
                write!(f, "An error happened while getting the API sbatch directory")
            },
        }
    }
}
