use crate::constants::MiddlewareError;
use model::establish_connection;
use uuid::Uuid;

pub fn insert(jobgroup: dto::jobgroups::InsertJobGroup) -> Result<dto::jobgroups::JobGroup, MiddlewareError> {
    let mut connection = establish_connection();
    match model::operations::jobgroups::insert(&mut connection, jobgroup.into()) {
        Ok(jobgroup) => Ok(jobgroup.into()),
        Err(err) => Err(MiddlewareError::ModelError(err))
    }
}

pub fn find_by_id(id: Uuid) -> Result<Option<dto::jobgroups::JobGroup>, MiddlewareError> {
    let mut connection = establish_connection();
    match model::operations::jobgroups::find_by_id(&mut connection, id) {
        Ok(jobgroup_found) => match jobgroup_found {
            Some(jobgroup) => Ok(Some(jobgroup.into())),
            None => Ok(None)
        },
        Err(err) => Err(MiddlewareError::ModelError(err))
    }
}

pub fn find_by_workspace(workspace_id: Uuid) -> Result<Vec<dto::jobgroups::JobGroup>, MiddlewareError> {
    let mut connection = establish_connection();
    match model::operations::jobgroups::find_by_workspace(&mut connection, workspace_id) {
        Ok(found_jobgroups) => {
            let mut jobgroups: Vec<dto::jobgroups::JobGroup> = Vec::new();
            for jobgroup in found_jobgroups {
                jobgroups.push(jobgroup.into());
            }
            Ok(jobgroups)
        }
        Err(err) => Err(MiddlewareError::ModelError(err))
    }
}

pub fn update(id: Uuid, jobgroup: dto::jobgroups::UpdateJobGroup) -> Result<usize, MiddlewareError> {
    let mut connection = establish_connection();
    match model::operations::jobgroups::update(&mut connection, id, jobgroup.into()) {
        Ok(jobgroup) => Ok(jobgroup),
        Err(err) => Err(MiddlewareError::ModelError(err))
    }
}

pub fn delete(id: Uuid) -> Result<usize, MiddlewareError> {
    let mut connection = establish_connection();
    match model::operations::jobgroups::delete(&mut connection, id) {
        Ok(jobgroup) => Ok(jobgroup),
        Err(err) => Err(MiddlewareError::ModelError(err))
    }
}
