use uuid::Uuid;
use rand::thread_rng;
use rand::distributions::{Alphanumeric, DistString};
use chrono::Utc;

use crate::constants::MiddlewareError;
use model::establish_connection;
use model::models::keys::NewKey;
use model::models::users::NewUser;
use model::operations::keys::*;
use model::operations::users::find_by_indentity;
use dto::keys::Key;
use dto::users::User;

pub fn generate_key(identity: Uuid) -> Result<Key, MiddlewareError> {
    let mut connection = establish_connection();
    // Check if a user with the given identity already exists
    let user = match find_by_indentity(&mut connection,identity.to_string()) {
        Ok(user_option) => {
            match user_option {
                Some(val) => val,
                None => {
                    // If it doesn't exists, we create it
                    let new_user = NewUser {
                        team: "test".to_string(),
                        identity: Some(identity.to_string())
                    };
                    match model::operations::users::insert(&mut connection, new_user) {
                        Ok(user) => user,
                        Err(err) => return Err(MiddlewareError::ModelError(err))
                    }
                }
            }
        },
        Err(err) => return Err(MiddlewareError::ModelError(err))
    };
    loop {
        // Generate random key
        let new_key: String = Alphanumeric.sample_string(&mut thread_rng(), 20);
        // If the key isn't already used or expired
        match  is_key_usable(&mut connection, new_key.clone()) {
            Ok(can_use) => {
                if can_use {
                    // Insert the key in the db, and return if it succeed
                    let new_insert_key: NewKey = NewKey {
                        user_id: user.id,
                        register_date: Utc::now().naive_utc(),
                        token: new_key,
                        status: "active".to_string()
                    };
                    match model::operations::keys::insert(&mut connection, new_insert_key) {
                        Ok(key) => {
                            return Ok(key.into())
                        },
                        Err(err) => {
                            return Err(MiddlewareError::ModelError(err))
                        }
                    }
                }
            },
            Err(err) => return Err(MiddlewareError::ModelError(err))
        }
    }
}

pub fn delete(key_id: Uuid) -> Result<usize, MiddlewareError> {
    let mut connection = establish_connection();
    match model::operations::keys::delete(&mut connection, key_id) {
        Ok(nb_keys_deleted) => Ok(nb_keys_deleted),
        Err(error) => Err(MiddlewareError::ModelError(error))
    }
}

pub fn get_key_associated_user(key: String) -> Result<User, MiddlewareError> {
    let mut connection = establish_connection();
    match model::operations::keys::get_key_associated_user(&mut connection, key) {
        Ok(user) => Ok(user.into()),
        Err(error) => Err(MiddlewareError::ModelError(error))
    }
}
