use crate::constants::MiddlewareError;
use dto::nodes::InsertNode;
use dto::nodes::Node;
use model::establish_connection;
use uuid::Uuid;

pub fn insert(node: InsertNode) -> Result<Node, MiddlewareError> {
    let mut connection = establish_connection();
    match model::operations::nodes::insert(&mut connection, node.into()) {
        Ok(node) => Ok(node.into()),
        Err(error) => Err(MiddlewareError::ModelError(error))
    }
}

pub fn find_by_id(id: Uuid) -> Result<Option<Node>, MiddlewareError> {
    let mut connection = establish_connection();
    match model::operations::nodes::find_by_id(&mut connection, id) {
        Ok(node_option) => match node_option {
            Some(node) => Ok(Some(node.into())),
            None => Ok(None)
        },
        Err(error) => Err(MiddlewareError::ModelError(error))
    }
}

pub fn update(id: Uuid, node: dto::nodes::UpdateNode) -> Result<usize, MiddlewareError> {
    let mut connection = establish_connection();
    match model::operations::nodes::update(&mut connection, id, node.into()) {
        Ok(node) => Ok(node),
        Err(error) => Err(MiddlewareError::ModelError(error))
    }
}

pub fn delete(id: Uuid) -> Result<usize, MiddlewareError> {
    let mut connection = establish_connection();
    match model::operations::nodes::delete(&mut connection, id) {
        Ok(node) => Ok(node),
        Err(error) => Err(MiddlewareError::ModelError(error))
    }
}

pub fn get_all_for_group(searched_jobgroup_id: Uuid) -> Result<Vec<Node>, MiddlewareError> {
    let mut connection = establish_connection();
    match model::operations::nodes::get_all_for_group(&mut connection, searched_jobgroup_id) {
        Ok(nodes) => {
            let mut result: Vec<Node> = Vec::new();
            for element in nodes {
                result.push(element.into())
            }
            Ok(result)
        }
        Err(error) => Err(MiddlewareError::ModelError(error))
    }
}
