use crate::constants::MiddlewareError;
use dto::parameters::*;
use model::establish_connection;
use uuid::Uuid;

pub fn insert(parameter: InsertParam) -> Result<Param, MiddlewareError> {
    let mut connection = establish_connection();
    match model::operations::parameters::insert(&mut connection, parameter.into()) {
        Ok(parameter) => Ok(parameter.into()),
        Err(err) => Err(MiddlewareError::ModelError(err))
    }
}

pub fn find_by_id(id: Uuid) -> Result<Option<Param>, MiddlewareError> {
    let mut connection = establish_connection();
    match model::operations::parameters::find_by_id(&mut connection, id) {
        Ok(parameter_option) => match parameter_option {
            Some(parameter) => Ok(Some(parameter.into())),
            None => Ok(None)
        },
        Err(err) => Err(MiddlewareError::ModelError(err))
    }
}

pub fn update(id: Uuid, parameter: UpdateParam) -> Result<usize, MiddlewareError> {
    let mut connection = establish_connection();
    match model::operations::parameters::update(&mut connection, id, parameter.into()) {
        Ok(nb_parameters_updated) => Ok(nb_parameters_updated),
        Err(err) => Err(MiddlewareError::ModelError(err))
    }
}

pub fn delete(id: Uuid) -> Result<usize, MiddlewareError> {
    let mut connection = establish_connection();
    match model::operations::parameters::delete(&mut connection, id) {
        Ok(nb_parameters_deleted) => Ok(nb_parameters_deleted),
        Err(err) => Err(MiddlewareError::ModelError(err))
    }
}

pub fn  find_by_job(searched_job_id: Uuid) -> Result<Vec<Param>, MiddlewareError> {
    let mut connection = establish_connection();
    match model::operations::parameters::find_by_job(&mut connection, searched_job_id) {
        Ok(found_parameters) => {
            let mut params: Vec<dto::parameters::Param> = Vec::new();
            for found_parameter in found_parameters {
                params.push(found_parameter.into());
            }
            Ok(params)
        }
        Err(err) => Err(MiddlewareError::ModelError(err))
    }
}
