use crate::constants::MiddlewareError;
use dto::projects::*;
use model::establish_connection;
use uuid::Uuid;

pub fn insert(project: InsertProject) -> Result<Project, MiddlewareError> {
    let mut connection = establish_connection();
    match model::operations::projects::insert(&mut connection, project.into()) {
        Ok(project) => Ok(project.into()),
        Err(error) => Err(MiddlewareError::ModelError(error))
    }
}

pub fn find_by_id(id: Uuid) -> Result<Option<Project>, MiddlewareError> {
    let mut connection = establish_connection();
    match model::operations::projects::find_by_id(&mut connection, id) {
        Ok(project) => match project {
            Some(project) => Ok(Some(project.into())),
            None => Ok(None)
        },
        Err(error) => Err(MiddlewareError::ModelError(error))
    }
}

pub fn find_by_user(user_id: Uuid) -> Result<Vec<Project>, MiddlewareError> {
    let mut connection = establish_connection();
    match model::operations::projects::find_by_user(&mut connection, user_id) {
        Ok(projects_found) => {
            let mut found_projects: Vec<dto::projects::Project> = Vec::new();
            for project in projects_found {
                found_projects.push(project.into());
            }
            Ok(found_projects)
        }
        Err(error) => Err(MiddlewareError::ModelError(error))
    }
}

pub fn update(id: Uuid, project: UpdateProject) -> Result<usize, MiddlewareError> {
    let mut connection = establish_connection();
    match model::operations::projects::update(&mut connection, id, project.into()) {
        Ok(nb_projects_updated) => Ok(nb_projects_updated),
        Err(error) => Err(MiddlewareError::ModelError(error))
    }
}

pub fn delete(id: Uuid) -> Result<usize, MiddlewareError> {
    let mut connection = establish_connection();
    match model::operations::projects::delete(&mut connection, id) {
        Ok(nb_projects_deleted) => Ok(nb_projects_deleted),
        Err(error) => Err(MiddlewareError::ModelError(error))
    }
}

pub fn find_by_team(team: String) -> Result<Vec<Project>, MiddlewareError> {
    let mut connection = establish_connection();
    match model::operations::projects::find_by_team(&mut connection, team) {
        Ok(projects) => {
            let mut final_list: Vec<Project> = Vec::new();
            for e in projects.into_iter() {
                final_list.push(e.into());
            }
            Ok(final_list)
        }
        Err(error) => Err(MiddlewareError::ModelError(error))
    }
}
