use crate::constants::MiddlewareError;
use dto::users::*;
use model::establish_connection;
use model::operations::users::get_all;
use uuid::Uuid;

pub fn insert(user: InsertUser) -> Result<User, MiddlewareError> {
    let mut connection = establish_connection();
    match model::operations::users::insert(&mut connection, user.into()) {
        Ok(user) => Ok(user.into()),
        Err(err) => Err(MiddlewareError::ModelError(err))
    }
}

pub fn list() -> Result<Vec<User>, MiddlewareError> {
    let mut connection = establish_connection();
    match get_all(&mut connection) {
        Ok(users) => {
            let mut users_vec: Vec<User> = Vec::new();
            for user in users {
                users_vec.push(user.into());
            }
            Ok(users_vec)
        }
        Err(err) => Err(MiddlewareError::ModelError(err))
    }
}

pub fn find_by_id(id: Uuid) -> Result<Option<User>, MiddlewareError> {
    let mut connection = establish_connection();
    match model::operations::users::find_by_id(&mut connection, id) {
        Ok(user_option) => match user_option {
            Some(user) => Ok(Some(user.into())),
            None => Ok(None)
        },
        Err(err) => Err(MiddlewareError::ModelError(err))
    }
}

pub fn find_by_team(team: String) -> Result<Vec<User>, MiddlewareError> {
    let mut connection = establish_connection();
    match model::operations::users::find_by_team(&mut connection, team) {
        Ok(found_users) => {
            let mut result: Vec<User> = Vec::new();
            for element in found_users {
                result.push(element.into())
            }
            Ok(result)
        }
        Err(err) => Err(MiddlewareError::ModelError(err))
    }
}

pub fn update(id: Uuid, user: UpdateUser) -> Result<usize, MiddlewareError> {
    let mut connection = establish_connection();
    match model::operations::users::update(&mut connection, id, user.into()) {
        Ok(updated_user) => Ok(updated_user),
        Err(err) => Err(MiddlewareError::ModelError(err))
    }
}

pub fn delete(id: Uuid) -> Result<usize, MiddlewareError> {
    let mut connection = establish_connection();
    match model::operations::users::delete(&mut connection, id) {
        Ok(users_deleted) => Ok(users_deleted),
        Err(err) => Err(MiddlewareError::ModelError(err))
    }
}
