use crate::constants::MiddlewareError;
use dto::workspaces::InsertWorkspace;
use dto::workspaces::UpdateWorkspace;
use dto::workspaces::Workspace;
use model::establish_connection;
use uuid::Uuid;

pub fn insert(workspace: InsertWorkspace) -> Result<Workspace, MiddlewareError> {
    let mut connection = establish_connection();
    match model::operations::workspaces::insert(&mut connection, workspace.into()) {
        Ok(user) => Ok(user.into()),
        Err(err) => Err(MiddlewareError::ModelError(err))
    }
}

pub fn find_by_id(id: Uuid) -> Result<Option<Workspace>, MiddlewareError> {
    let mut connection = establish_connection();
    match model::operations::workspaces::find_by_id(&mut connection, id) {
        Ok(found_workspace) => match found_workspace {
            Some(workspace) => Ok(Some(workspace.into())),
            None => Ok(None)
        },
        Err(err) => Err(MiddlewareError::ModelError(err))
    }
}

pub fn find_by_project(searched_project_id: Uuid) -> Result<Vec<Workspace>, MiddlewareError> {
    let mut connection = establish_connection();
    match model::operations::workspaces::find_by_project(&mut connection, searched_project_id) {
        Ok(updated_workspace) => {
            let mut result: Vec<Workspace> = Vec::new();
            for element in updated_workspace {
                result.push(element.into());
            }
            Ok(result)
        }
        Err(err) => Err(MiddlewareError::ModelError(err))
    }
}

pub fn update(id: Uuid, workspace: UpdateWorkspace) -> Result<usize, MiddlewareError> {
    let mut connection = establish_connection();
    match model::operations::workspaces::update(&mut connection, id, workspace.into()) {
        Ok(updated_workspace) => Ok(updated_workspace),
        Err(err) => Err(MiddlewareError::ModelError(err))
    }
}

pub fn delete(id: Uuid) -> Result<usize, MiddlewareError> {
    let mut connection = establish_connection();
    match model::operations::workspaces::delete(&mut connection, id) {
        Ok(deleted_workspaces_count) => Ok(deleted_workspaces_count),
        Err(err) => Err(MiddlewareError::ModelError(err))
    }
}
