use crate::schema::jobgroups::dsl::jobgroups;
use crate::schema::jobgroups::*;
use diesel::prelude::*;
use uuid::Uuid;

use crate::models::jobgroups::*;

pub fn insert(
    conn: &mut PgConnection,
    new_jobgroup: NewJobGroup
) -> Result<JobGroup, diesel::result::Error> {
    use crate::schema::jobgroups;
    let table_to_update = diesel::insert_into(jobgroups::table);
    table_to_update.values(&new_jobgroup).get_result(conn)
}

pub fn update(
    conn: &mut PgConnection,
    jobgroup_id: uuid::Uuid,
    jobgroup: UpdateJobGroup
) -> Result<usize, diesel::result::Error> {
    diesel::update(jobgroups.filter(id.eq(jobgroup_id)))
        .set(jobgroup)
        .execute(conn)
}

pub fn delete(conn: &mut PgConnection, jobgroup_id: Uuid) -> Result<usize, diesel::result::Error> {
    diesel::delete(jobgroups.filter(id.eq(jobgroup_id))).execute(conn)
}

pub fn find_by_id(conn: &mut PgConnection, jobgroup_id: Uuid) -> QueryResult<Option<JobGroup>> {
    jobgroups
        .filter(id.eq(jobgroup_id))
        .first::<JobGroup>(conn)
        .optional()
}

pub fn find_by_workspace(
    conn: &mut PgConnection,
    searched_workspace_id: uuid::Uuid
) -> QueryResult<Vec<JobGroup>> {
    let workspace = match crate::operations::workspaces::find_by_id(conn, searched_workspace_id) {
        Ok(found_workspace) => match found_workspace {
            Some(val) => val,
            None => return Err(diesel::result::Error::NotFound)
        },
        Err(err) => return Err(err)
    };
    let workspace_groups = match <JobGroup as BelongingToDsl<
        &crate::models::workspaces::Workspace
    >>::belonging_to(&workspace)
    .load::<JobGroup>(conn)
    {
        Ok(result) => result,
        Err(err) => return Err(err)
    };
    Ok(workspace_groups)
}
