use crate::schema::jobs::dsl::jobs;
use crate::schema::jobs::*;
use diesel::prelude::*;
use uuid::Uuid;

use crate::models::jobgroupassociations::JobGroupAssociation;
use crate::models::jobs::*;

pub fn insert(conn: &mut PgConnection, new_job: NewJob) -> Result<Job, diesel::result::Error> {
    use crate::schema::jobs;
    let table_to_update = diesel::insert_into(jobs::table);
    table_to_update.values(&new_job).get_result(conn)
}

pub fn update(
    conn: &mut PgConnection,
    job_id: uuid::Uuid,
    job: UpdateJob
) -> Result<usize, diesel::result::Error> {
    diesel::update(jobs.filter(id.eq(job_id)))
        .set(job)
        .execute(conn)
}

pub fn delete(conn: &mut PgConnection, job_id: Uuid) -> Result<usize, diesel::result::Error> {
    diesel::delete(jobs.filter(id.eq(job_id))).execute(conn)
}

pub fn find_by_id(conn: &mut PgConnection, job_id: Uuid) -> QueryResult<Option<Job>> {
    jobs.filter(id.eq(job_id)).first::<Job>(conn).optional()
}

pub fn get_all_for_group(
    conn: &mut PgConnection,
    searched_group_id: uuid::Uuid
) -> QueryResult<Vec<(Job, i32)>> {
    let group = match crate::operations::jobgroups::find_by_id(conn, searched_group_id) {
        Ok(found_group) => match found_group {
            Some(val) => val,
            None => return Err(diesel::result::Error::NotFound)
        },
        Err(err) => return Err(err)
    };
    let group_associations = match <JobGroupAssociation as BelongingToDsl<
        &crate::models::jobgroups::JobGroup
    >>::belonging_to(&group)
    .load::<JobGroupAssociation>(conn)
    {
        Ok(result) => result,
        Err(err) => return Err(err)
    };
    let mut group_jobs = Vec::new();
    for association in group_associations.into_iter() {
        let job = match crate::operations::jobs::find_by_id(conn, association.job_id) {
            Ok(found_job) => match found_job {
                Some(val) => val,
                None => return Err(diesel::result::Error::NotFound)
            },
            Err(err) => return Err(err)
        };
        group_jobs.push((job, association.position));
    }
    Ok(group_jobs)
}
