use crate::schema::keys::dsl::keys;
use crate::schema::keys::*;
use diesel::prelude::*;
use uuid::Uuid;

use crate::models::keys::*;
use crate::models::users::User;

pub fn insert(conn: &mut PgConnection, new_key: NewKey) -> Result<Key, diesel::result::Error> {
    use crate::schema::keys;
    let table_to_update = diesel::insert_into(keys::table);
    table_to_update.values(&new_key).get_result(conn)
}

pub fn delete(conn: &mut PgConnection, key_id: Uuid) -> Result<usize, diesel::result::Error> {
    diesel::delete(keys.filter(id.eq(key_id))).execute(conn)
}

pub fn find_by_id(conn: &mut PgConnection, key_id: Uuid) -> QueryResult<Option<Key>> {
    keys.filter(id.eq(key_id)).first::<Key>(conn).optional()
}

pub fn get_key_associated_user(conn: &mut PgConnection, key: String) -> QueryResult<User> {
    let user_key = match keys.filter(token.eq(key)).first::<Key>(conn).optional() {
        Ok(found_key) => match found_key {
            Some(val) => val,
            None => return Err(diesel::result::Error::NotFound)
        },
        Err(err) => return Err(err)
    };
    let user = match crate::operations::users::find_by_id(conn, user_key.user_id) {
        Ok(found_user) => match found_user {
            Some(val) => val,
            None => return Err(diesel::result::Error::NotFound)
        },
        Err(err) => return Err(err)
    };
    Ok(user)
}

pub fn is_key_usable(conn: &mut PgConnection, new_key: String) -> Result<bool, diesel::result::Error>  {
    let result = match keys.filter(token.eq(new_key)).first::<Key>(conn).optional() {
        Ok(Some(_)) => false,
        Ok(None) => true,
        Err(err) => return Err(err)
    };
    Ok(result)
}
