use crate::schema::nodes::dsl::nodes;
use crate::schema::nodes::*;
use diesel::prelude::*;
use uuid::Uuid;

use crate::models::nodes::*;

pub fn insert(conn: &mut PgConnection, new_node: NewNode) -> Result<Node, diesel::result::Error> {
    use crate::schema::nodes;
    let table_to_update = diesel::insert_into(nodes::table);
    table_to_update.values(&new_node).get_result(conn)
}

pub fn update(
    conn: &mut PgConnection,
    node_id: uuid::Uuid,
    node: UpdateNode
) -> Result<usize, diesel::result::Error> {
    diesel::update(nodes.filter(id.eq(node_id)))
        .set(node)
        .execute(conn)
}

pub fn delete(conn: &mut PgConnection, node_id: Uuid) -> Result<usize, diesel::result::Error> {
    diesel::delete(nodes.filter(id.eq(node_id))).execute(conn)
}

pub fn find_by_id(conn: &mut PgConnection, node_id: Uuid) -> QueryResult<Option<Node>> {
    nodes.filter(id.eq(node_id)).first::<Node>(conn).optional()
}

pub fn get_all_for_group(
    conn: &mut PgConnection,
    searched_jobgroup_id: Uuid
) -> QueryResult<Vec<Node>> {
    let jobgroup = match crate::operations::jobgroups::find_by_id(conn, searched_jobgroup_id) {
        Ok(found_jobgroup) => match found_jobgroup {
            Some(val) => val,
            None => return Err(diesel::result::Error::NotFound)
        },
        Err(err) => return Err(err)
    };
    let jobgroup_nodes =
        match <Node as BelongingToDsl<&crate::models::jobgroups::JobGroup>>::belonging_to(&jobgroup)
            .load::<Node>(conn)
        {
            Ok(result) => result,
            Err(err) => return Err(err)
        };
    Ok(jobgroup_nodes)
}
