use crate::schema::parameters::dsl::parameters;
use crate::schema::parameters::*;
use diesel::prelude::*;
use uuid::Uuid;

use crate::models::parameters::*;

pub fn insert(
    conn: &mut PgConnection,
    new_param: NewParameter
) -> Result<Parameter, diesel::result::Error> {
    use crate::schema::parameters;
    let table_to_update = diesel::insert_into(parameters::table);
    table_to_update.values(&new_param).get_result(conn)
}

pub fn update(
    conn: &mut PgConnection,
    param_id: uuid::Uuid,
    param: UpdateParameter
) -> Result<usize, diesel::result::Error> {
    diesel::update(parameters.filter(id.eq(param_id)))
        .set(param)
        .execute(conn)
}

pub fn delete(conn: &mut PgConnection, param_id: Uuid) -> Result<usize, diesel::result::Error> {
    diesel::delete(parameters.filter(id.eq(param_id))).execute(conn)
}

pub fn find_by_id(conn: &mut PgConnection, param_id: Uuid) -> QueryResult<Option<Parameter>> {
    parameters
        .filter(id.eq(param_id))
        .first::<Parameter>(conn)
        .optional()
}

pub fn find_by_job(conn: &mut PgConnection, searched_job_id: Uuid) -> QueryResult<Vec<Parameter>> {
    let job = match crate::operations::jobs::find_by_id(conn, searched_job_id) {
        Ok(found_job) => match found_job {
            Some(val) => val,
            None => return Err(diesel::result::Error::NotFound)
        },
        Err(err) => return Err(err)
    };
    let job_params =
        match <Parameter as BelongingToDsl<&crate::models::jobs::Job>>::belonging_to(&job)
            .load::<Parameter>(conn)
        {
            Ok(result) => result,
            Err(err) => return Err(err)
        };
    Ok(job_params)
}
