use crate::schema::projects::dsl::projects;
use crate::schema::projects::*;
use diesel::prelude::*;
use uuid::Uuid;

use crate::models::projects::*;

pub fn insert(
    conn: &mut PgConnection,
    new_project: NewProject
) -> Result<Project, diesel::result::Error> {
    use crate::schema::projects;
    let table_to_update = diesel::insert_into(projects::table);
    table_to_update.values(&new_project).get_result(conn)
}

pub fn update(
    conn: &mut PgConnection,
    project_id: Uuid,
    project: UpdateProject
) -> Result<usize, diesel::result::Error> {
    diesel::update(projects.filter(id.eq(project_id)))
        .set(project)
        .execute(conn)
}

pub fn delete(conn: &mut PgConnection, project_id: Uuid) -> Result<usize, diesel::result::Error> {
    diesel::delete(projects.filter(id.eq(project_id))).execute(conn)
}

pub fn find_by_id(conn: &mut PgConnection, project_id: Uuid) -> QueryResult<Option<Project>> {
    projects
        .filter(id.eq(project_id))
        .first::<Project>(conn)
        .optional()
}

pub fn find_by_user(conn: &mut PgConnection, searched_user_id: Uuid) -> QueryResult<Vec<Project>> {
    projects
        .filter(user_id.eq(searched_user_id))
        .load::<Project>(conn)
}

pub fn find_by_team(conn: &mut PgConnection, searched_team: String) -> QueryResult<Vec<Project>> {
    let team = match crate::operations::users::find_by_team(conn, searched_team) {
        Ok(users) => users,
        Err(err) => return Err(err)
    };
    let mut team_projects = Vec::new();
    for user in team.into_iter() {
        let mut user_projects =
            match <Project as BelongingToDsl<&crate::models::users::User>>::belonging_to(&user)
                .load::<Project>(conn)
            {
                Ok(result) => result,
                Err(err) => return Err(err)
            };
        team_projects.append(&mut user_projects);
    }
    Ok(team_projects)
}
