use crate::schema::users::dsl::users;
use crate::schema::users::*;
use diesel::prelude::*;
use uuid::Uuid;

use crate::models::users::*;

pub fn insert(conn: &mut PgConnection, new_user: NewUser) -> Result<User, diesel::result::Error> {
    use crate::schema::users;
    let table_to_update = diesel::insert_into(users::table);
    table_to_update.values(&new_user).get_result(conn)
}

pub fn update(
    conn: &mut PgConnection,
    user_id: uuid::Uuid,
    user: UpdateUser
) -> Result<usize, diesel::result::Error> {
    diesel::update(users.filter(id.eq(user_id)))
        .set(user)
        .execute(conn)
}

pub fn delete(conn: &mut PgConnection, user_id: Uuid) -> Result<usize, diesel::result::Error> {
    diesel::delete(users.filter(id.eq(user_id))).execute(conn)
}

pub fn find_by_id(conn: &mut PgConnection, user_id: Uuid) -> QueryResult<Option<User>> {
    users.filter(id.eq(user_id)).first::<User>(conn).optional()
}

pub fn find_by_team(conn: &mut PgConnection, searched_team: String) -> QueryResult<Vec<User>> {
    users.filter(team.eq(searched_team)).load::<User>(conn)
}

pub fn find_by_indentity(conn: &mut PgConnection, searched_identity: String) -> QueryResult<Option<User>> {
    users.filter(identity.eq(searched_identity)).first::<User>(conn).optional()
}

pub fn get_all(conn: &mut PgConnection) -> QueryResult<Vec<User>> {
    users.load::<User>(conn)
}
