use crate::schema::workspaces::dsl::workspaces;
use crate::schema::workspaces::*;
use diesel::prelude::*;
use uuid::Uuid;

use crate::models::workspaces::*;

pub fn insert(
    conn: &mut PgConnection,
    new_workspace: NewWorkspace
) -> Result<Workspace, diesel::result::Error> {
    use crate::schema::workspaces;
    let table_to_update = diesel::insert_into(workspaces::table);
    table_to_update.values(&new_workspace).get_result(conn)
}

pub fn update(
    conn: &mut PgConnection,
    workspace_id: uuid::Uuid,
    workspace: UpdateWorkspace
) -> Result<usize, diesel::result::Error> {
    diesel::update(workspaces.filter(id.eq(workspace_id)))
        .set(workspace)
        .execute(conn)
}

pub fn delete(conn: &mut PgConnection, workspace_id: Uuid) -> Result<usize, diesel::result::Error> {
    diesel::delete(workspaces.filter(id.eq(workspace_id))).execute(conn)
}

pub fn find_by_id(conn: &mut PgConnection, workspace_id: Uuid) -> QueryResult<Option<Workspace>> {
    workspaces
        .filter(id.eq(workspace_id))
        .first::<Workspace>(conn)
        .optional()
}

pub fn find_by_project(
    conn: &mut PgConnection,
    searched_project_id: Uuid
) -> QueryResult<Vec<Workspace>> {
    let user_project = match crate::operations::projects::find_by_id(conn, searched_project_id) {
        Ok(found_project) => match found_project {
            Some(project) => project,
            None => return Err(diesel::result::Error::NotFound)
        },
        Err(err) => return Err(err)
    };
    let p_workspaces =
        match <Workspace as BelongingToDsl<&crate::models::projects::Project>>::belonging_to(
            &user_project
        )
        .load::<Workspace>(conn)
        {
            Ok(result) => result,
            Err(err) => return Err(err)
        };
    Ok(p_workspaces)
}
