table! {
    jobgroupassociations (id) {
        id -> Uuid,
        jobgroup_id -> Uuid,
        job_id -> Uuid,
        position -> Int4,
    }
}

table! {
    jobgroups (id) {
        id -> Uuid,
        input_job_id -> Uuid,
        workspace_id -> Uuid,
    }
}

table! {
    jobs (id) {
        id -> Uuid,
        name -> Varchar,
        output -> Varchar,
        job_type -> Varchar,
        version -> Varchar,
        software -> Varchar,
        slurm_id -> Varchar,
    }
}

table! {
    keys (id) {
        id -> Uuid,
        user_id -> Uuid,
        register_date -> Timestamp,
        token -> Varchar,
        status -> Varchar,
    }
}

table! {
    nodes (id) {
        id -> Uuid,
        jobgroup_id -> Uuid,
        x -> Float8,
        y -> Float8,
    }
}

table! {
    parameters (id) {
        id -> Uuid,
        job_id -> Uuid,
        name -> Varchar,
        value -> Varchar,
    }
}

table! {
    projects (id) {
        id -> Uuid,
        name -> Varchar,
        user_id -> Uuid,
    }
}

table! {
    users (id) {
        id -> Uuid,
        team -> Varchar,
        identity -> Nullable<Varchar>,
    }
}

table! {
    workspaces (id) {
        id -> Uuid,
        name -> Varchar,
        project_id -> Uuid,
    }
}

joinable!(jobgroupassociations -> jobgroups (jobgroup_id));
joinable!(jobgroupassociations -> jobs (job_id));
joinable!(jobgroups -> jobs (input_job_id));
joinable!(jobgroups -> workspaces (workspace_id));
joinable!(keys -> users (user_id));
joinable!(nodes -> jobgroups (jobgroup_id));
joinable!(parameters -> jobs (job_id));
joinable!(projects -> users (user_id));
joinable!(workspaces -> projects (project_id));

allow_tables_to_appear_in_same_query!(
    jobgroupassociations,
    jobgroups,
    jobs,
    keys,
    nodes,
    parameters,
    projects,
    users,
    workspaces,
);
