use crate::auth::Authorization;
use rocket::State;
use rocket::http::Status;
use rocket::serde::{json::Json, uuid::Uuid};

use dto::config::*;
use middleware::misc::config::*;

use adapters::adapters::get_conf_by_uuid;
use adapters::constants::Configurations;

use rocket_okapi::openapi;

/// # Returns the directory structure of the config folder
///
/// Returns status code 500 if it failed
///
/// Returns status code 200 with the directory structure of the config folder
#[openapi]
#[get("/config/list")]
pub fn get_config_list(configs: &State<Configurations>, _claims: Authorization) -> Result<Json<Configs>, (Status, String)> {
    let config_list: Configs = get_conf_list(configs.inner().clone());
    if config_list.softwares.is_empty() {
        Err((Status::InternalServerError, "The directory structure is empty".to_string()))
    } else {
        Ok(Json(config_list))
    }
}

/// # Returns the config identified by its id
///
/// Returns status code 404 if the config was not found
///
/// Returns status code 200 with the config identified by its id
#[openapi]
#[get("/config/<id>")]
pub fn get_config(id: Uuid, configs: &State<Configurations>, _claims: Authorization) -> Result<Json<JobConfCommand>, (Status, String)> {
    match get_conf_by_uuid(id, configs.inner().clone()) {
       Some(val) => Ok(Json(JobConfCommand::from(val))),
       None => Err((Status::NotFound, "Config doesn't exists, unable to retrieve it".to_string()))
    }
}
