use dto::users::User;
use rocket::http::Status;
use rocket::serde::{json::Json, uuid::Uuid};

use middleware::models::keys::*;

use crate::auth::Authorization;
use dto::keys::*;
use rocket_okapi::openapi;

//
//Code 200 for success
//Code 500 for failure
#[openapi]
#[post("/")]
pub fn new_key(auth: Authorization) -> Result<Json<Key>, (Status, String)> {
    let sub: Uuid = match auth.jwt_claims {
        Some(val) => val.sub,
        None => return Err((Status::InternalServerError, "Error in JWT claims sub retrieving".to_string()))
    };
    match generate_key(sub) {
        Ok(new_key) => Ok(Json(new_key)),
        Err(error) => Err((Status::InternalServerError, error.to_string()))
    }
}

#[openapi]
#[delete("/<id>")]
pub fn delete_key(id: Uuid, _auth: Authorization) -> Result<(Status, String), (Status, String)> {
    let parsed_id = uuid::Uuid::parse_str(id.to_string().as_str()).unwrap();
    match delete(parsed_id) {
        Ok(deleted_rows_count) => match deleted_rows_count {
            0 => Err((Status::NotFound, "No row deleted".to_string())),
            _ => Ok((Status::Ok, format!("{} row deleted", deleted_rows_count)))
        },
        Err(error) => Err((Status::InternalServerError, error.to_string()))
    }
}

#[openapi]
#[get("/<key>/user")]
pub fn get_key_associated_user(key: String, _auth: Authorization) -> Result<Json<User>, (Status, String)> {
    match middleware::models::keys::get_key_associated_user(key) {
        Ok(found_user) => Ok(Json(found_user)),
        Err(error) => Err((Status::InternalServerError, error.to_string()))
    }
}
