use rocket::http::Status;
use rocket::local::blocking::Client;
use rocket::routes;

mod common;
use common::*;
use dto::keys::Key;
use dto::users::User;


#[test]
fn test_api_new_key() {
    use router::routes::keys::new_key;
    
    initialize_env();
    let jwt_token = initialize_jwt_token();
    let rocket = rocket::build().mount("/keys", routes![new_key]);
    let client = Client::tracked(rocket).unwrap();
    let auth_header = rocket::http::Header::new("Authorization", "Bearer ".to_owned() + &jwt_token);
    let login_response = client
        .post("/keys/")
        .header(auth_header)
        .dispatch();
    assert_eq!(login_response.status(), Status::Ok);
    assert!(login_response.body().is_some());
}

#[test]
fn test_api_new_key_forbidden() {
    use router::routes::keys::new_key;
    
    initialize_env();
    let rocket = rocket::build().mount("/keys", routes![new_key]);
    let client = Client::tracked(rocket).unwrap();
    let login_response = client
        .post("/keys/")
        .dispatch();
    assert_eq!(login_response.status(), Status::Unauthorized);
}

#[test]
fn test_api_get_key_associated_user() {
    use router::routes::keys::{new_key, get_key_associated_user};
    
    initialize_env();
    let jwt_token = initialize_jwt_token();
    let login_header = rocket::http::Header::new("Authorization", "Bearer ".to_owned() + &jwt_token);
    let rocket = rocket::build().mount("/keys", routes![new_key, get_key_associated_user]);
    let client = Client::tracked(rocket).unwrap();
    let login_response = client
        .post("/keys/")
        .header(login_header)
        .dispatch();
    let api_key = login_response.into_json::<Key>().unwrap().key;
    let auth_header = rocket::http::Header::new("Authorization", "APIKey ".to_owned() + &api_key);
    let user_response = client
        .get(format!("/keys/{}/user", api_key))
        .header(auth_header)
        .dispatch();
    assert_eq!(user_response.status(), Status::Ok);
    let json_response = user_response.into_json::<User>().unwrap();
    assert_eq!(json_response.team, "test".to_string());
    assert_eq!(json_response.identity, "f3a142eb-0fad-4eb8-b0d8-2d2dfdccff02".to_string());
}

#[test]
fn test_api_get_key_error() {
    use router::routes::keys::{new_key, get_key_associated_user};
    
    initialize_env();
    let jwt_token = initialize_jwt_token();
    let login_header = rocket::http::Header::new("Authorization", "Bearer ".to_owned() + &jwt_token);
    let rocket = rocket::build().mount("/keys", routes![new_key, get_key_associated_user]);
    let client = Client::tracked(rocket).unwrap();
    let login_response = client
        .post("/keys/")
        .header(login_header)
        .dispatch();
    let api_key = login_response.into_json::<Key>().unwrap().key;
    let auth_header = rocket::http::Header::new("Authorization", "APIKey ".to_owned() + &api_key);
    let user_response = client
        .get(format!("/keys/{}/user", "AzErTyuI45sdfvoejc5s"))
        .header(auth_header)
        .dispatch();
    assert_eq!(user_response.status(), Status::InternalServerError);
}

#[test]
fn test_api_get_key_forbidden() {
    use router::routes::keys::{new_key, get_key_associated_user};
    
    initialize_env();
    let rocket = rocket::build().mount("/keys", routes![new_key, get_key_associated_user]);
    let client = Client::tracked(rocket).unwrap();
    let user_response = client
        .get(format!("/keys/{}/user", "AzErTyuI45sdfvoejc5s"))
        .dispatch();
    assert_eq!(user_response.status(), Status::Unauthorized);
}
